@@ -4,8 +4,8 @@ module Bundler
   # Represents metadata from when the Bundler gem was built.
   module BuildMetadata
     # begin ivars
-    @built_at = "2020-01-05".freeze
-    @git_commit_sha = "32a4159325".freeze
+    @built_at = "2020-12-14".freeze
+    @git_commit_sha = "b98d6b2035".freeze
     @release = true
     # end ivars
 
@@ -29,19 +29,11 @@ module Bundler
 
       # If Bundler has been installed without its .git directory and without a
       # commit instance variable then we can't determine its commits SHA.
-      git_dir = File.join(File.expand_path("../../..", __FILE__), ".git")
+      git_dir = File.join(File.expand_path("../../../..", __FILE__), ".git")
       if File.directory?(git_dir)
         return @git_commit_sha = Dir.chdir(git_dir) { `git rev-parse --short HEAD`.strip.freeze }
       end
 
-      # If Bundler is a submodule in RubyGems, get the submodule commit
-      git_sub_dir = File.join(File.expand_path("../../../..", __FILE__), ".git")
-      if File.directory?(git_sub_dir)
-        return @git_commit_sha = Dir.chdir(git_sub_dir) do
-          `git ls-tree --abbrev=8 HEAD bundler`.split(/\s/).fetch(2, "").strip.freeze
-        end
-      end
-
       @git_commit_sha ||= "unknown"
     end
 
